==> qbittorrent_4.3.3.orig.tar.xz: qbittorrent-4.3.3/src/base/utils/bytearray.cpp <==
/*
 * Bittorrent Client using Qt and libtorrent.
 * Copyright (C) 2018  Mike Tzou (Chocobo1)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * In addition, as a special exception, the copyright holders give permission to
 * link this program with the OpenSSL project's "OpenSSL" library (or with
 * modified versions of it that use the same license as the "OpenSSL" library),
 * and distribute the linked executables. You must obey the GNU General Public
 * License in all respects for all of the code used other than "OpenSSL".  If you
 * modify file(s), you may extend this exception to your version of the file(s),
 * but you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 */

#include "bytearray.h"

#include <QByteArray>
#include <QVector>

QVector<QByteArray> Utils::ByteArray::splitToViews(const QByteArray &in, const QByteArray &sep, const QString::SplitBehavior behavior)
{
    if (sep.isEmpty())
        return {in};

    QVector<QByteArray> ret;
    ret.reserve((behavior == QString::KeepEmptyParts)
                ? (1 + (in.size() / sep.size()))
                : (1 + (in.size() / (sep.size() + 1))));
    int head = 0;
    while (head < in.size())
    {
        int end = in.indexOf(sep, head);
        if (end < 0)
            end = in.size();

        // omit empty parts
        const QByteArray part = QByteArray::fromRawData((in.constData() + head), (end - head));
        if (!part.isEmpty() || (behavior == QString::KeepEmptyParts))
            ret += part;

        head = end + sep.size();
    }

    return ret;
}

const QByteArray Utils::ByteArray::midView(const QByteArray &in, const int pos, const int len)
{
    if ((pos < 0) || (pos >= in.size()) || (len == 0))
        return {};

    const int validLen = ((len < 0) || (pos + len) >= in.size())
            ? in.size() - pos
            : len;
    return QByteArray::fromRawData(in.constData() + pos, validLen);
}

==> qbittorrent_4.6.3.orig.tar.xz: qbittorrent-4.6.3/src/base/utils/bytearray.cpp <==
/*
 * Bittorrent Client using Qt and libtorrent.
 * Copyright (C) 2023  Vladimir Golovnev <glassez@yandex.ru>
 * Copyright (C) 2018  Mike Tzou (Chocobo1)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * In addition, as a special exception, the copyright holders give permission to
 * link this program with the OpenSSL project's "OpenSSL" library (or with
 * modified versions of it that use the same license as the "OpenSSL" library),
 * and distribute the linked executables. You must obey the GNU General Public
 * License in all respects for all of the code used other than "OpenSSL".  If you
 * modify file(s), you may extend this exception to your version of the file(s),
 * but you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 */

#include "bytearray.h"

#include <QByteArray>
#include <QVector>

QVector<QByteArray> Utils::ByteArray::splitToViews(const QByteArray &in, const QByteArray &sep, const Qt::SplitBehavior behavior)
{
    if (sep.isEmpty())
        return {in};

    QVector<QByteArray> ret;
    ret.reserve((behavior == Qt::KeepEmptyParts)
                ? (1 + (in.size() / sep.size()))
                : (1 + (in.size() / (sep.size() + 1))));
    int head = 0;
    while (head < in.size())
    {
        int end = in.indexOf(sep, head);
        if (end < 0)
            end = in.size();

        // omit empty parts
        const QByteArray part = QByteArray::fromRawData((in.constData() + head), (end - head));
        if (!part.isEmpty() || (behavior == Qt::KeepEmptyParts))
            ret += part;

        head = end + sep.size();
    }

    return ret;
}

const QByteArray Utils::ByteArray::midView(const QByteArray &in, const int pos, const int len)
{
    if ((pos < 0) || (pos >= in.size()) || (len == 0))
        return {};

    const int validLen = ((len < 0) || (pos + len) >= in.size())
            ? in.size() - pos
            : len;
    return QByteArray::fromRawData(in.constData() + pos, validLen);
}

QByteArray Utils::ByteArray::toBase32(const QByteArray &in)
{
    const char alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
    const char padchar = '=';

    const qsizetype inSize = in.size();

    auto tmp = QByteArray((inSize + 4) / 5 * 8, Qt::Uninitialized);
    qsizetype inIndex = 0;
    char *out = tmp.data();
    while (inIndex < inSize)
    {
        // encode 5 bytes at a time
        qsizetype inPadLen = 5;
        int64_t chunk = 0;
        while (inPadLen > 0)
        {
            chunk |= static_cast<int64_t>(static_cast<uchar>(in.data()[inIndex++])) << (--inPadLen * 8);
            if (inIndex == inSize)
                break;
        }

        const int outCharCounts[] = {8, 7, 5, 4, 2};
        for (int i = 7; i >= 0; --i)
        {
            if (i >= (8 - outCharCounts[inPadLen]))
            {
                const int shift = (i * 5);
                const int64_t mask = static_cast<int64_t>(0x1f) << shift;
                const int charIndex = (chunk & mask) >> shift;
                *out++ = alphabet[charIndex];
            }
            else
            {
                *out++ = padchar;
            }
        }
    }

    return tmp;
}
